--[[
	Target: In this example script, it is shown how resource-constrained project scheduling
			using the time minimization objective can be done using P2 Engine. 
			This script is an experiment written on page 117 of "Project Management with
			Dynamic Scheduling - Baseline Scheduling, Risk Analysis and Project Control" 
			(Vanhoucke, page 117 and also figures 7.1, 7.2, 7.3 and 7.4 and table 7.2). 
	More information: www.p2engine.com
	Start with: main_procedure_dynamic_scheduling_chapter7_part1
	Uses the following subprocedure(s): "subprocedure_create_objects" and "subprocedure_manual_simulation"
	Classes: P2Project, P2Schedule, P2Reader and P2Resource
	Last update: August 29, 2012
--]]

-- Define your path where you unzip your downloaded file from www.p2engine.com (MUST BE CHANGED BY THE USER!!)
MY_PATH=string.format('/Users/mariovanhoucke/Documenten van Mario/Lua Scripts/tutorial/')

-- Other paths are defined automatically (no need to change these paths!)
MY_SCRIPTS=MY_PATH..'DynamicSchedulingExamples/'
MY_GENERAL=MY_PATH..'general/'
dofile(MY_GENERAL..'subprocedure_create_objects.lua')
dofile(MY_GENERAL..'subprocedure_read_files.lua')

function main_procedure_dynamic_scheduling_chapter7_part1()
	-- Create the objects
	create_project_object()
	create_schedule_object()
	create_resource_object()

	-- Read the project network of chapter 2 from a file 
	read_a_patterson_file(MY_SCRIPTS..'project_example_dynamic_scheduling_table7.2.rcp')
	CPM=MySchedule:get_project_pd()
	
	-- Write the resource information to the screen: you will see that the resource ID starts at zero (and not one)
	MyResource:write_resource_data()
	
	-- Schedule the project without resources (CPM) and with resource (RCP) with various techniques
	io.write('\n\nProject duration (excluding resource constraints)')
	io.write('\n\tCritical path length = ',CPM)
	
	-- Resource constrained project scheduling: include resources one by one and schedule them twice
	--		First scheduling approach: priority rule (shortest processing time)
	--		Second scheduling approach: intensive search (truncated after 1000 schedules)
	-- 	Note: A similar experiment has been done at page 115 of measuring time
	io.write('\nProject duration (including resource constraints)')
	truncation_limit=1000
	for resID=1,MyResource:get_number_of_resources() do
		include_resources_in_leveling(resID)
		io.write('\n  Including ',resID,' resource(s)')
		RCP_PD=MySchedule:level_resources(1,0,1,"CallBackResourceLeveling")
		io.write('\n\t\tProject duration (priority rule)    = ',RCP_PD)
		RCP_PD=MySchedule:level_resources(1,-1,1,"CallBackResourceLeveling")
		io.write('\n\t\tProject duration (insensive search) = ',RCP_PD)
	end

	-- Write the starting times to the screen (figure 7.4)
	io.write('\n\nWrite the starting times of the last resource scheduling run (including all resources)')
	for act=1,MySchedule:get_number_of_activities() do
		d=MySchedule:get_duration(act)
		s=MySchedule:get_resource_scheduled_start_time(act)
		io.write('\nActivity ',act,' starts at ',s)
		io.write(' and finishes at time ',s+d)
	end
	-- Delete the objects
	delete_resource_object()
	delete_schedule_object()
	delete_project_object()
	release_all_memory()
end

function include_resources_in_leveling(resPtr)
	-- All resources with ID < resPtr are included in the leveling process
	for res=0,MyResource:get_number_of_resources()-1 do
		if res<resPtr then MySchedule:include_in_leveling(res,1)
		else MySchedule:include_in_leveling(res,0) 
		end
	end
end

function CallBackResourceLeveling(p1,nrTrials,p3)
--[[
	This procedure is called at every run (1 to nrruns) of a resource_leveling (intensive search)
	and allows the user to interrupt the search based on conditions or to write data to a file 
	for every run.
	visit http://www.p2engine.com/p2schedule/level_resources
--]]
	if nrTrials>=truncation_limit then 
--		io.write('\n\tTotal number of solutions found without over-allocations = ',p3)
		return 1 
	end
--	io.write('\t',nrTrials)
	return 0
end

main_procedure_dynamic_scheduling_chapter7_part1()
