--[[
	Target: In this example script, it is shown how P2 Engine can reproduce all the data 
			for the three project examples of the book written by Mario Vanhoucke,
			"Integrated Project Control using ProTrack and P2 Engine" 
	More information: www.p2engine.com
	Start with: main_procedure_controlling_three_projects
	Uses the following subprocedure(s): "subprocedure_create_objects", "subprocedure_read_files", and "subprocedure_manual_simulation"
	Classes: P2Project, P2Schedule, P2Reader and P2Tracking
	Last update: August 27, 2012
--]]

-- Define your path where you unzip your downloaded file from www.p2engine.com (MUST BE CHANGED BY THE USER!!)
MY_PATH=string.format('/Users/mariovanhoucke/Documenten van Mario/Lua Scripts/tutorial/')
-- tijdelijk pad
MY_DATA=string.format('/Users/mariovanhoucke/Documenten van Mario/Research Folder/Academic Topics/Boeken/Control EVM_SRA/pictures/projecten/') 
-- Other paths are defined automatically (no need to change these paths!)
MY_SCRIPTS=MY_PATH..'NewBookExamples/'
MY_GENERAL=MY_PATH..'general/'
--dofile(MY_SCRIPTS..'subprocedure_manual_simulation.lua')
dofile(MY_PATH..'MeasuringTimeExamples/'..'subprocedure_manual_simulation.lua')
--dofile(MY_PATH..'MeasuringTimeExamples/'..'example_chapter5_schedule_risk_analysis.lua') --run a schedule risk analysis
dofile(MY_GENERAL..'subprocedure_read_files.lua')
dofile(MY_GENERAL..'subprocedure_create_objects.lua')
dofile(MY_SCRIPTS..'example_projects_info.lua')
dofile(MY_SCRIPTS..'schedule_risk_analysis.lua')

-- Call this script using the terminal in mac with the following command:
-- /Users/mariovanhoucke/Documenten\ van\ Mario/XCode\ Projecten/OR-AS\ research/research/p2engine/build/Debug/p2 '/Users/mariovanhoucke/Documenten\ van\ Mario/Lua Scripts/tutorial/NewBookExamples/example_project1_new_book.lua' 1 (or 2 or 3 for project 2 or 3)
-- of beter: "/Users/mariovanhoucke/Documenten van Mario/XCode Projecten/OR-AS research/research/p2engine/build/Debug/p2" "/Users/mariovanhoucke/Documenten van Mario/Lua Scripts/tutorial/NewBookExamples/example_project1_new_book.lua" 1 (or 2 or 3 for project 2 or 3)
-- A similar call can be done in Windows

function main_procedure_controlling_three_projects()
	-- Create the three objects
	create_project_object()
	create_schedule_object()
	create_generator_object() --needed to calculate the SP of the projects
	create_tracking_object()

	-- Read the project ID (1, 2 or 3) as input	
	local projectID = tonumber(arg[1])
	filename = 'project_'..arg[1]..'.rcp'
	io.write(filename)

	-- Read the project networks (project_1.rcp or project_2.rcp or project_3.rcp)
	read_a_patterson_file(MY_DATA..filename)
	label_activities(projectID)
	add_costs(projectID)
	
	-- Write baseline schedule
	write_baseline_schedule_information()
	
	-- Imitate project progress (manually)
	imitate_progress(projectID)

	-- Write progress information
	write_latex_tables() 					-- call after "imitate_progress"
	write_baseline_and_progress_schedule()  -- call after "imitate_progress"

	-- Create an EVM data table (similar to table 2.2 of the book "Measuring Time")
	create_EVM_table2_2()
	
	-- Schedule risk analysis
	create_simulator_object()
	
	MySchedule:put_batch_calculator_on()
	nract=MySchedule:get_number_of_activities()
	for act=1,nract do
		dur=MySchedule:get_duration(act)
		MySchedule:change_duration(act,dur*5)
	end
	MySchedule:put_batch_calculator_off()
	
	nrruns=20
	run_a_schedule_risk_analysis(nrruns)
	print_SRA_output_table()
	delete_simulator_object()


	-- Delete all objects
	delete_tracking_object()
	delete_generator_object()
	delete_schedule_object()
	delete_project_object()
	release_all_memory()
end

function add_costs(projectID)
	-- Assign costs per time unit to activities (cost per unit = total cost / activity duration)
	-- In the example, the costs per unit are assigned in a for loop using an array of costs
	if projectID==1 then
		cost_per_time_unit={1000,7500,7500,1500,1300,1300,900,1000,1500,1500,3000,1000,1000,1000,1000,2000,5000}
	elseif projectID==2 then
		cost_per_time_unit={5000,2500,700,1500,5000,1500,5000,6500,6000,500,1100,1900,600,9000,1500,100,5000}
	else
		cost_per_time_unit={10000,10000,30000,10000,20000,25000,10000,15000,25000,5000,4000,10000,10000,15000,3000,10000,20000}		
	end
	
	for act,cost in ipairs(cost_per_time_unit) do
		--io.write("\nAssign cost to ",act)
		MySchedule:set_variable_cost(act,cost)
	end
end

function imitate_progress(projectID)
	-- Manual input of progress data: real start times and durations for each activity
	local RealStT_delta={}
	local RealDur_delta={}
	if projectID==1 then
		RealStT_delta={0,0,0,0,0,0,-1,-1,0,0,0,-1,-1,0,2,0,0} 			--status: OK (delay)
		RealDur_delta={0,1,1,-1,2,0,1,0,-2,0,1,0,1,0,2,0,0}	 			--status: OK
	elseif projectID==2 then
		RealStT_delta={0,0,0,-2,-1,0,-1,-3,-1,0,-1,-1,-3,-1,1,-3,-1}  	--status: OK (on time, p-factor << 1)
		RealDur_delta={0,1,1,-1,2,0,1,0,-2,0,1,0,1,0,2,0,0}   		  	--status: OK
	else
		RealStT_delta={0,0,0,-2,-2,-5,-5,-5,-5,-5,-5,-5,-5,-5,-5,-6,-6} --status: OD (early, p-factor < 1)
		RealDur_delta={0,1,1,-1,2,0,1,0,-2,0,-1,0,1,0,0,0,0}   			--status: OK
	end
	
	nract=MySchedule:get_number_of_activities()
	RealStartTimes={}
	RealDurations={}
	RealCost={}
	
	--[[ Step 1. Calculate the real duration and real start (no overlaps possible)
	-- Real Duration = Baseline Duration + afwijking
	-- Real Cost = Baseline Cost met lineare afwijking
	-- Real Start = temporarily change the baseline schedule with new duration, calculate the new ESS 
					and new start times (deviations from ESS) and restore the original baseline schedule --]]
	
	local baselineDur={}
	for act=1,nract do
		baselineDur[act]=MySchedule:get_duration(act)
		RealDurations[act]=baselineDur[act]+RealDur_delta[act]
		if RealDurations[act]<1 then RealDurations[act]=1 end
		MySchedule:change_duration(act,RealDurations[act])
	end
	MySchedule:earliest_start_schedule_with_float_factor(0)

	-- get the earliest start times of the activities with the real durations (with overlaps)
	for act=1,nract do
		RealStartTimes[act]=MySchedule:get_start(act)+RealStT_delta[act]
		if RealStartTimes[act]<0 then RealStartTimes[act]=0 end
		RealCost[act]=MySchedule:get_variable_cost(act)*RealDurations[act] -- voorlopig lineair
	end

	-- restore baseline data
	MySchedule:put_batch_calculator_on()
	for act=1,nract do
		MySchedule:change_duration(act,baselineDur[act])
	end
	MySchedule:put_batch_calculator_off()
	MySchedule:earliest_start_schedule_with_float_factor(0)

	-- simulate progress given data above	
	simulate_progress_manually(RealStartTimes,RealDurations)
end	



function create_EVM_table2_2()
	io.write('\nThis table is similar to table 2.2 of Measuring Time (available in text file)\n')
	-- Create a table with the EVM key metrics and performance measures (including p-factor) and write to file
	-- (see table 2.2 of the book "Measuring Time")

	outfilename = 'project_'..arg[1]..'.out'

	-- Create an empty file
	output_file=string.format(MY_DATA..outfilename)
	file=io.open(output_file,"w")

	-- Get the number of activities, the planned project duration and the number of tracking periods (which is equal to the real duration since each tracking period increases with 1 time unit in this example)
	nract=MySchedule:get_number_of_activities()					-- 17 in this example
	nrbaselineper=MySchedule:get_project_pd()					
	nrtrackper=MyTracking:get_number_of_tracking_periods()		-- Hoeveel? Per week?

	-- Write header
	file:write('Day')
	for period=1,math.max(nrtrackper,nrbaselineper) do
		file:write('\t',period)
	end

	file:write('\n\t\t\t\t\t\t\t\tTraditional Earned Value Calculations')

	-- Write the Earned Value (EV)
	file:write('\nEV\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_ev(period)))
	end

	-- Write the Actual Cost (AC)
	file:write('\nAC\t')
	for period=1,nrtrackper do
		file:write(MyTracking:get_project_ac(period),'\t')
	end

	-- Write the Planned Value (PV)
	file:write('\nPV\t')
	for period=1,nrtrackper do
		file:write(MyTracking:get_project_pv(period),'\t')
	end
	
	--[[ In case the project finishes early, no tracking periods have been created 
	   between the finish and the PD and hence the PV curve is not known in this interval.
	   Therefore, new fictitious tracking periods are generated and only the PV is calculated --]]
	if nrtrackper<nrbaselineper then
		local timePtr=nrtrackper+1
		while timePtr<=nrbaselineper do
			periodID=MyTracking:add_tracking_period(timePtr)
			MyTracking:construct_evm_report(periodID)
			file:write(MyTracking:get_project_pv(periodID),'\t')
			timePtr=timePtr+1
		end	
	end

	file:write('\n')

	-- Write the EVM performance measures
	file:write('\nSV\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_sv(period)))
	end
	file:write('\nCV\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.0f\t',MyTracking:get_project_cv(period)))
	end	
	file:write('\nSPI\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_spi(period)))
	end
	file:write('\nCPI\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_cpi(period)))
	end
	
	file:write('\n')
	file:write('\nES\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_es(period)))
	end
	file:write('\nSV(t)\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_svt(period)))
	end
	file:write('\nSPI(t)\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_spit(period)))
	end
	
	file:write('\n\t\t\t\t\t\t\t\tEffective Earned Value Calculations')
	file:write('\np-factor\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_p_factor(period)))
		--io.write(string.format('%2.2f\t',MyTracking:get_project_p_factor(period)))
	end
	
	-- Effective EVM is not incorporated in P2 Engine but can be easily calculated as illustrated along the following lines
	-- Note: Try to avoid to program procedures yourself when available in P2 Engine

	-- Set the estimated portion of EV(r) that is usable and required no rework
	RPercent=0.9
		
	-- Calculate and write the effective Earned Value (EV(e))
	ev_e={}
	file:write('\nEV(e)\t')
	for period=1,nrtrackper do
		ev=MyTracking:get_project_ev(period)
		p=MyTracking:get_project_p_factor(period)
		ev_e[period]=ev*p+RPercent*(1-p)*ev
		file:write(string.format('%2.2f\t',ev_e[period]))
	end

	-- Write the EVM performance measures
	file:write('\nSV(e)\t')
	for period=1,nrtrackper do
		sv_e=ev_e[period]-MyTracking:get_project_pv(period)
		file:write(string.format('%2.2f\t',sv_e))
	end
	file:write('\nCV(e)\t')
	for period=1,nrtrackper do
		cv_e=ev_e[period]-MyTracking:get_project_ac(period)
		file:write(string.format('%2.2f\t',cv_e))
	end	
	file:write('\nSPI(e)\t')
	for period=1,nrtrackper do
		spi_e=ev_e[period]/MyTracking:get_project_pv(period)
		file:write(string.format('%2.2f\t',spi_e))
	end
	file:write('\nCPI(e)\t')
	for period=1,nrtrackper do
		cpi_e=ev_e[period]/MyTracking:get_project_ac(period)
		file:write(string.format('%2.2f\t',cpi_e))
	end

	-- Calculate and write the effective Earned Schedule (ES(e))
	es_e={}
	for period=1,nrtrackper do
		timePtr=0
		for t=1,nrtrackper do
			pvt=MyTracking:get_project_pv(t)
			if pvt>=ev_e[period] then 
				timePtr=t-1
				break
			end
		end	
		if timePtr==0 then
			pvt=0
		else
			pvt=MyTracking:get_project_pv(timePtr)
		end	
		pvtplus=MyTracking:get_project_pv(timePtr+1)
		es_e[period]=timePtr+(ev_e[period]-pvt)/(pvtplus-pvt)
	end

	file:write('\n\nES(e)\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',es_e[period]))
	end

	file:write('\nSV(t)(e)\t')
	for period=1,nrtrackper do
		sv_e=es_e[period]-MyTracking:get_status_date(period)
		file:write(string.format('%2.2f\t',sv_e))
	end

	file:write('\nSPI(t)(e)\t')
	for period=1,nrtrackper do
		spi_e=es_e[period]/MyTracking:get_status_date(period)
		file:write(string.format('%2.2f\t',spi_e))
	end
	
	file:write('\n\nCost Forecasting (EAC)')
	file:write('\nReal Cost\t',MyTracking:get_real_project_cost())
	file:write('\nPF=1\tPF=CPI\tPF=SPI\tPF=SPI(t)\tPF=SCI\tPF=SCI(t)\tPF=w(SPI,CPI)\tPF=w(SPI(t),CPI)')
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\n%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f',EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI))
	end

	file:write('\n\n\n\nTime Forecasting (EAC(t))')
	file:write('\nReal Time\t',MyTracking:get_real_project_duration())
	file:write('\nPV1\tPV2\tPV3\tED1\tED2\tED3\tES1\tES2\tES3')
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\n%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f',EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3))
	end

	-- write to screen (in other format)
	file:write('\n\nCOST FORECASTING')	
	file:write('\nPF=1')	
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PF1))--,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI))
	end

	file:write('\nPF=CPI')
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PFCPI))
	end

	file:write('\nPF=SPI')
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PFSPI))
	end

	file:write('\nPF=SPI(t)')
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PFSPIT))
	end
	
	file:write('\nPF=SCI')	
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PFSCI))
	end

	file:write('\nPF=SCI(t)')
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PFSCIT))
	end

	file:write('\nPF=w(SPI,CPI)')
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PFweightedSPICPI))
	end

	file:write('\nPF=w(SPI(t),CPI)')
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PFweightedSPITCPI))
	end
	
	file:write('\n\nTIME FORECASTING')	
	file:write('\nPV-PF=1')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_PV1))
	end

	file:write('\nPV-PF=SPI')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_PV2))
	end

	file:write('\nPV-PF=SCI')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_PV3))
	end

	file:write('\nED-PF=1')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_ED1))
	end

	file:write('\nED-PF=SPI')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_ED2))
	end

	file:write('\nED-PF=SCI')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_ED3))
	end

	file:write('\nES-PF=1')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_ES1))
	end

	file:write('\nES-PF=SPI(t)')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_ES2))
	end

	file:write('\nES-PF=SCI(t)')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_ES3))
	end

	-- Close the file
	file:flush()
	file:close()
end

main_procedure_controlling_three_projects()
