function run_a_schedule_risk_analysis(nrruns)
	-- reset_simulation_object: connect to new schedule (no need to make a new P2Simulator object)	
	MySimulator:reset_simulation_object(MySchedule)

	-- toewijzen risicoprofielen
	Triang_prob={0,100,0}

	-- Assign the triangular distributions to the project activities (Monte-Carlo simulation)
	-- For each activity: (a,m,b) = (planned duration - %, planned duration, planned duration + %)
	triangular_probability={0,100,0}
	MySimulator:create_monte_carlo_object()
	for act=1,MySchedule:get_number_of_activities() do
		MySimulator:set_number_advanced_distributions(act,3)
		lower=20	-- percentage below the planned duration: if lower > upper: project on average early
		upper=20	-- percentage below the planned duration: if lower < upper: project on average late
		triangular_values={math.floor((1-(lower/100))*MySchedule:get_duration(act)),math.floor(MySchedule:get_duration(act)),math.ceil((1+(upper/100))*MySchedule:get_duration(act))}
		for pnt=1,3 do
			MySimulator:set_advanced_distributions(act,pnt,triangular_values[pnt],triangular_probability[pnt])
		end
	end
	
	-- Set time unit (= length of each period during project progress simulation)
	time_unit=8
	MySimulator:set_periodic_time_unit(time_unit)
	
	-- Run the simulations
	MySimulator:set_callback("sim_callback")
	MySimulator:run_simulation(nrruns,11,1,1,0,0)

end

function sim_callback(p1,p2,p3)
--[[
	Procedure set in "set_callback"
	This procedure is called at every run (1 to nrruns) of each simulation and is a powerful tool
	to interrupt simulations based on conditions or to write data to the screen or a file for every simulation.

	For more information:
	visit http://www.p2engine.com/p2simulator/set_callback 
	or look at other example scripts available at http://www.p2engine.com/examples
	
	In this example, the tracking data is generated in a tracking object for each run. 
	All the data of this tracking object can then be written to a text file and can be analysed. 
	This is exactly how PhD students at OR&S work (see www.projectmanagement.ugent.be).
	In the current example, the following information is written to the screen (but never used since it was only for illustrative purposes)
		- The number of tracking periods
		- The real (simulated) duration of the project 
		- The real (simulated) cost of the project
		- The real (simulated) duration of each activity
		- Project is early, on time, late
	Obviously, a lot of other data can be generated. 
	Imagine what databases you can create to analyse the data and do research!
--]]

	io.write('\nSimulation run ',p1,'   ')
	MyTracking:reset_tracking_object()
	MySimulator:write_output_to_tracking_list(MyTracking)
		
	nrperiods=MyTracking:get_number_of_tracking_periods()
	io.write('\t'..nrperiods)	
	RD=MyTracking:get_real_project_duration()
	RC=MyTracking:get_real_project_cost()
	io.write("\t",RD)
	io.write("\t",RC)
	for act=1,MySchedule:get_number_of_activities() do
		realdur=MyTracking:get_actual_duration(act,nrperiods)
		io.write(string.format('\t%d',realdur))
	end
	PD=MySchedule:get_project_pd()
	if PD<RD then io.write('\tlate')
		elseif PD>RD then io.write('\tearly') 
		else io.write('\ton time')
	end
	return 0
end

function print_SRA_output_table()	
	io.write('\nTime sensitivity table')
	io.write('\nActivity ID\tCI\tSI\tSSI\tCRI(r)\tCRI(rho)\tCRI(tau)')
	for act=1,MySchedule:get_number_of_activities() do
		sensitivity_matrix={MySimulator:get_activity_time_sensitivity(act)}
		io.write('\n',act)
		for i=1,6 do
			io.write(string.format('\t%8.2f',sensitivity_matrix[i]))
		end
	end	
	io.write('\nCost sensitivity table')
	io.write('\nActivity ID\t  CRI(r)\tCRI(rho)\tCRI(tau)')
	for act=1,MySchedule:get_number_of_activities() do
		sensitivity_matrix={MySimulator:get_activity_cost_sensitivity(act)}
		io.write('\n',act)
		for i=1,3 do
			io.write(string.format('\t%8.2f',sensitivity_matrix[i]))
		end
	end	
end	
