--[[
	Target: This subroutine performs a manual simulation of a project with given start times
			and durations of the activities in progress
	More information: www.p2engine.com
	Main procedure: none (this is a subprocedure used by other main procedures)
	Classes: P2Tracking
	Last update: August 25, 2012
--]]

function simulate_progress_manually(RealStart,RealDuration)
	
	-- Reset the Tracking object: Calling this procedure is crucial since the tracking object was created BEFORE the project and schedule data was added.
	-- An alternative to this procedure is to create the tracking object here
	-- As a rule of thumb: always use "reset_tracking_object" at the beginning of a simulation to reset and clear all tracking periods
	MyTracking:reset_tracking_object()
	
	-- Create the real execution of the project (see figure 1.9 of the book "Measuring Time")
--	RealDuration = {6,11,1,4,3,2,8,8,5,4}
--	RealStart = {0,0,0,6,1,4,6,10,11,6}

	-- Construct period tracking periods (18 in total with each period an increase of 1 time unit (timePtr))
	local timePtr=1
	local projectEnded=false
	while projectEnded==false do
		nrFinish=0
		periodID=MyTracking:add_tracking_period(timePtr)
		nract=MySchedule:get_number_of_activities()
		for act=1,nract do
			if timePtr<RealStart[act] then 
				-- Activity: not started yet
				PC=0 
				AD=0
				AS=99999999
			elseif timePtr>=RealStart[act]+RealDuration[act] then 
				-- Activity: finished
				PC=1 
				AD=RealDuration[act]
				AS=RealStart[act]
				nrFinish=nrFinish+1
			else
				-- Activity: busy 
				PC=(timePtr-RealStart[act])/RealDuration[act] 
				AD=timePtr-RealStart[act]
				AS=RealStart[act]
			end
			
			MyTracking:set_actual_start(act,AS,periodID)
			MyTracking:set_percentage_completed(act,PC,periodID)
			MyTracking:set_actual_duration(act,AD,periodID)
			AC=MyTracking:get_planned_actual_cost(act,periodID)	-- Only possible when AD has been set!
			MyTracking:set_actual_cost(act,AC,periodID)
		end
		
		-- Construct an EVM report for this tracking period (automatically calculates al possible EVM output measures)
		MyTracking:construct_evm_report(periodID)
		
		timePtr=timePtr+1
		if nrFinish==nract then projectEnded=true end
    end
end


